//
//  Concentration.swift
//  Concentration
//
//  Created by Kohei Koja on 2018/08/22.
//  Copyright © 2018年 Kohei Koja. All rights reserved.
//

import Foundation

class Concentration {
    
    private(set) var cards = [Card]()
    private var appearedCards = [Card]()
    
    let emojiThemes = [
        0: "🐶🐱🐭🐼🐽🐔🐵🦄🦁",
        1: "😀😂😍🤪😎😡🤢😇🤩",
        2: "🦇🤡🙀👿🎃👻🍭🍬💀",
        3: "⚽️🏀🏈⚾️🎾🏐🏉🎱🏓",
        4: "🍎🍐🍉🍒🥝🍆🥦🥒🥐",
        5: "👍👎✌️🤟🖐🤙🤝👊👌"
    ]
    
    var emojiChoices = ""

    var flipCount = 0
    var scoreCount = 0

    private var indexOfOneAndOnlyFaceUpCard: Int? {
        get {
            /*
             thanks to extension of protocol (oneAndOnly below) and
             closures, we could shorten code here.
             */
            return cards.indices.filter { return cards[$0].isFaceUp }.oneAndOnly
//            var foundIndex: Int?
//            for index in cards.indices {
//                if cards[index].isFaceUp {
//                    if foundIndex == nil {
//                        foundIndex = index
//                    } else {
//                        return nil
//                    }
//                }
//            }
//            return foundIndex
        }
        
        // we actually dont need newValue which is special argument for set
        set(newValue) {
            for index in cards.indices {
                cards[index].isFaceUp = (index == newValue)
            }
        }
    }
    
    func chooseCard(at index: Int) {
        assert(cards.indices.contains(index), "Concentraion.chooseCard(at: \(index)): chosen index is not in the cards")
        
        if !cards[index].isMatched {
            if !cards[index].isFaceUp {
                flipCount += 1
            }
            
            if let matchIndex = indexOfOneAndOnlyFaceUpCard, matchIndex != index {
                // check if cards match
                if cards[matchIndex] == cards[index] {
                    cards[matchIndex].isMatched = true
                    cards[index].isMatched = true
                    scoreCount += 2
                } else { // when cards are not matched
//                    if appearedCards.contains(cards[matchIndex]) {
//                        scoreCount -= 1
//                    } else {
//                        appearedCards.append(cards[matchIndex])
//                    }
                    
                    // MARK: the 3 lines code below is optional
                    if !appearedCards.contains(cards[matchIndex]) {
                        appearedCards.append(cards[matchIndex])
                    }
                    
                    if appearedCards.contains(cards[index]) {
                        scoreCount -= 1
                    } else {
                        appearedCards.append(cards[index])
                    }
                }
                
                cards[index].isFaceUp = true
            } else {
                // either no cards or 2 cards face up
                indexOfOneAndOnlyFaceUpCard = index
            }
        }
    }
    
    init(numberOfPairOfCards: Int) {
        assert(numberOfPairOfCards > 0, "Concentraion.init(\(numberOfPairOfCards)): You must have at least one pair of cards")
        for _ in 1...numberOfPairOfCards {
            let card = Card()
            cards += [card, card]
        }
        // TODO Shuffle the cards
        cards.shuffle()
    }
    
    
}

extension MutableCollection {
    /// Shuffles the contents of this collection.
    mutating func shuffle() {
        let c = count
        guard c > 1 else { return }
        
        for (firstUnshuffled, unshuffledCount) in zip(indices, stride(from: c, to: 1, by: -1)) {
            // Change `Int` in the next line to `IndexDistance` in < Swift 4.1
            let d: IndexDistance = numericCast(arc4random_uniform(numericCast(unshuffledCount)))
            let i = index(firstUnshuffled, offsetBy: d)
            swapAt(firstUnshuffled, i)
        }
    }
}

extension Collection {
    var oneAndOnly: Element? {
        return count == 1 ? first : nil
    }
    /*
       "count" and "first" are Collection method
       so we can use these in implementations of Collection var
    */
}


