//
//  ViewController.swift
//  Concentration
//
//  Created by Kohei Koja on 2018/08/21.
//  Copyright © 2018年 Kohei Koja. All rights reserved.
//

import UIKit

class ConcentrationViewController: UIViewController {
    
    private lazy var game = Concentration(numberOfPairOfCards: numberOfPairOfCards)
    
    // this variable is read only
    var numberOfPairOfCards: Int {
        return (cardButtons.count + 1) / 2
    }
    
    private func updateFlipCountLabel() {
        let attributes: [NSAttributedStringKey: Any] = [
            .strokeWidth : -5.0,
            .strokeColor : #colorLiteral(red: 1, green: 0.5763723254, blue: 0, alpha: 1),
        ]
        let attributedString = NSAttributedString(string: "Flips: \(game.flipCount)", attributes: attributes)
        flipCountLabel.attributedText = attributedString
    }
    
    private func updateScoreCountLabel() {
        let attributes: [NSAttributedStringKey: Any] = [
            .strokeWidth : -5.0,
            .strokeColor : #colorLiteral(red: 1, green: 0.5763723254, blue: 0, alpha: 1)
        ]
        let attributedString = NSAttributedString(string: "Scores: \(game.scoreCount)", attributes: attributes)
        scoreCountLabel.attributedText = attributedString
    }

    @IBOutlet private weak var flipCountLabel: UILabel! {
        didSet {
            updateFlipCountLabel()
        }
    }

    @IBOutlet weak var scoreCountLabel: UILabel! {
        didSet {
            updateScoreCountLabel()
        }
    }
    
    @IBOutlet private var cardButtons: [UIButton]!
    
    @IBAction private func createNewGame(_ sender: UIButton) {
        game = Concentration(numberOfPairOfCards: numberOfPairOfCards)
        updateViewFromModel()
    }
    
    // MARK: Handle card touching behaviour
    
    @IBAction private func touchCard(_ sender: UIButton) {
        if let cardNumber = cardButtons.index(of: sender) {
            game.chooseCard(at: cardNumber)
            updateViewFromModel()
        } else {
            print("chosen card is not in cardButtons")
        }
        
    }
    
    private func updateViewFromModel() {
        for index in cardButtons.indices {
            let button = cardButtons[index]
            let card = game.cards[index]
            if card.isFaceUp {
                button.setTitle(emoji(for: card), for: UIControlState.normal)
                button.backgroundColor = #colorLiteral(red: 1, green: 1, blue: 1, alpha: 1)
            } else {
                button.setTitle("", for: UIControlState.normal)
                button.backgroundColor = card.isMatched ? #colorLiteral(red: 1, green: 0.5763723254, blue: 0, alpha: 0) : #colorLiteral(red: 1, green: 0.5763723254, blue: 0, alpha: 1)
            }
        }
        updateFlipCountLabel()
        updateScoreCountLabel()
    }
    

    var theme: String? {
        didSet {
            theme = theme ?? ""
            emoji = [:]
            updateViewFromModel()
        }
    }
    private var emoji = [Card:String]()
    
    private func emoji(for card: Card) -> String {
        if game.emojiChoices.isEmpty {
            chooseEmojiTheme()
        }
        if emoji[card] == nil, game.emojiChoices.count > 0 {
            let randomStringIndex = game.emojiChoices.index(game.emojiChoices.startIndex, offsetBy: game.emojiChoices.count.arc4random)
            emoji[card] = String(game.emojiChoices.remove(at: randomStringIndex))
        } 
        
        return emoji[card] ?? "?"
    }
    
    private func chooseEmojiTheme(){
        let randomIndex = game.emojiThemes.count.arc4random
        game.emojiChoices = game.emojiThemes[randomIndex]!
    }
    
}


extension Int {
    var arc4random: Int {
        if self > 0 {
            return Int(arc4random_uniform(UInt32(self)))
        } else if self < 0 {
            return -Int(arc4random_uniform(UInt32(abs(self))))
        } else {
            return 0
        }
    }
}

